@@ -26,7 +26,8 @@ supported in modules in L{storm.databases}.
 """
 
 from storm.expr import Expr, State, compile
-from storm.tracer import trace
+# Circular import: imported at the end of the module.
+# from storm.tracer import trace
 from storm.variables import Variable
 from storm.exceptions import (
     ClosedError, ConnectionBlockedError, DatabaseError, DisconnectionError,
@@ -224,9 +225,9 @@ class Connection(object):
             raise ClosedError("Connection is closed")
         if self._blocked:
             raise ConnectionBlockedError("Access to connection is blocked")
-        self._ensure_connected()
         if self._event:
             self._event.emit("register-transaction")
+        self._ensure_connected()
         if isinstance(statement, Expr):
             if params is not None:
                 raise ValueError("Can't pass parameters with expressions")
@@ -352,7 +353,7 @@ class Connection(object):
             else:
                 self._state = STATE_CONNECTED
 
-    def is_disconnection_error(self, exc):
+    def is_disconnection_error(self, exc, extra_disconnection_errors=()):
         """Check whether an exception represents a database disconnection.
 
         This should be overridden by backends to detect whichever
@@ -362,10 +363,14 @@ class Connection(object):
 
     def _check_disconnect(self, function, *args, **kwargs):
         """Run the given function, checking for database disconnections."""
+        # Allow the caller to specify additional exception types that
+        # should be treated as possible disconnection errors.
+        extra_disconnection_errors = kwargs.pop(
+            'extra_disconnection_errors', ())
         try:
             return function(*args, **kwargs)
-        except Error, exc:
-            if self.is_disconnection_error(exc):
+        except Exception, exc:
+            if self.is_disconnection_error(exc, extra_disconnection_errors):
                 self._state = STATE_DISCONNECTED
                 self._raw_connection = None
                 raise DisconnectionError(str(exc))
@@ -458,3 +463,6 @@ def create_database(uri):
                             None, None, [""])
         factory = module.create_from_uri
     return factory(uri)
+
+# Deal with circular import.        
+from storm.tracer import trace
