@@ -116,15 +116,24 @@ class NotSupportedError(DatabaseError):
 class DisconnectionError(OperationalError):
     pass
 
+
 class TimeoutError(StormError):
     """Raised by timeout tracers when remining time is over."""
 
-    def __init__(self, statement, params):
+    def __init__(self, statement, params, message=None):
         self.statement = statement
         self.params = params
+        self.message = message
 
     def __str__(self):
-        return "%r, %r" % (self.statement, self.params)
+	return ', '.join(
+            [repr(element) for element in
+             (self.message, self.statement, self.params)
+             if element is not None])
+
+
+class ConnectionBlockedError(StormError):
+    """Raised when an attempt is made to use a blocked connection."""
 
 
 def install_exceptions(module):
