@@ -79,10 +79,10 @@ class Compile(object):
 
         Use this as:
 
-        @compile.when(TypeA, TypeB)
-        def compile_type_a_or_b(compile, expr, state):
-            ...
-            return "THE COMPILED SQL STATEMENT"
+            >>> @compile.when(TypeA, TypeB)
+            >>> def compile_type_a_or_b(compile, expr, state):
+            >>>     ...
+            >>>     return "THE COMPILED SQL STATEMENT"
         """
         return _when(self, types)
 
@@ -656,10 +656,13 @@ class Select(Expr):
 @compile.when(Select)
 def compile_select(compile, select, state):
     tokens = ["SELECT "]
-    if select.distinct:
-        tokens.append("DISTINCT ")
     state.push("auto_tables", [])
     state.push("context", COLUMN)
+    if select.distinct:
+        tokens.append("DISTINCT ")
+        if isinstance(select.distinct, (tuple, list)):
+            tokens.append(
+                "ON (%s) " % compile(select.distinct, state, raw=True))
     tokens.append(compile(select.columns, state))
     tables_pos = len(tokens)
     parameters_pos = len(state.parameters)
@@ -797,7 +800,8 @@ class Column(ComparableExpr):
     @ivar variable_factory: Factory producing C{Variable} instances typed
         according to this column.
     """
-    __slots__ = ("name", "table", "primary", "variable_factory")
+    __slots__ = ("name", "table", "primary", "variable_factory",
+                 "compile_cache", "compile_id")
 
     def __init__(self, name=Undef, table=Undef, primary=False,
                  variable_factory=None):
@@ -805,6 +809,8 @@ class Column(ComparableExpr):
         self.table = table
         self.primary = int(primary)
         self.variable_factory = variable_factory or Variable
+        self.compile_cache = None
+        self.compile_id = None
 
 @compile.when(Column)
 def compile_column(compile, column, state):
@@ -816,11 +822,17 @@ def compile_column(compile, column, state):
             alias = state.aliases.get(column)
             if alias is not None:
                 return compile(alias.name, state, token=True)
-        return compile(column.name, state, token=True)
+        if column.compile_id != id(compile):
+            column.compile_cache = compile(column.name, state, token=True)
+            column.compile_id = id(compile)
+        return column.compile_cache
     state.push("context", COLUMN_PREFIX)
     table = compile(column.table, state, token=True)
     state.pop()
-    return "%s.%s" % (table, compile(column.name, state, token=True))
+    if column.compile_id != id(compile):
+        column.compile_cache = compile(column.name, state, token=True)
+        column.compile_id = id(compile)
+    return "%s.%s" % (table, column.compile_cache)
 
 @compile_python.when(Column)
 def compile_python_column(compile, column, state):
@@ -867,14 +879,20 @@ class FromExpr(Expr):
 
 
 class Table(FromExpr):
-    __slots__ = ("name",)
+    __slots__ = ("name", "compile_cache", "compile_id")
 
     def __init__(self, name):
         self.name = name
+        self.compile_cache = None
+        self.compile_id = None
+
 
 @compile.when(Table)
 def compile_table(compile, table, state):
-    return compile(table.name, state, token=True)
+    if table.compile_id != id(compile):
+        table.compile_cache = compile(table.name, state, token=True)
+        table.compile_id = id(compile)
+    return table.compile_cache
 
 
 class JoinExpr(FromExpr):
@@ -1286,6 +1304,31 @@ class Coalesce(NamedFunc):
     name = "COALESCE"
 
 
+class Row(NamedFunc):
+    __slots__ = ()
+    name = "ROW"
+
+
+class Cast(FuncExpr):
+    """A representation of C{CAST} clauses. e.g., C{CAST(bar AS TEXT)}."""
+    __slots__ = ("column", "type")
+    name = "CAST"
+
+    def __init__(self, column, type):
+        """Create a cast of C{column} as C{type}."""
+        self.column = column
+        self.type = type
+
+
+@compile.when(Cast)
+def compile_cast(compile, cast, state):
+    """Compile L{Cast} expressions."""
+    state.push("context", EXPR)
+    column = compile(cast.column, state)
+    state.pop()
+    return "CAST(%s AS %s)" % (column, cast.type)
+
+
 # --------------------------------------------------------------------
 # Prefix and suffix expressions
 
