@@ -167,12 +167,7 @@ class Store(object):
 
         primary_values = tuple(var.get(to_db=True) for var in primary_vars)
         obj_info = self._alive.get((cls_info.cls, primary_values))
-        if obj_info is not None:
-            if obj_info.get("invalidated"):
-                try:
-                    self._validate_alive(obj_info)
-                except LostObjectError:
-                    return None
+        if obj_info is not None and not obj_info.get("invalidated"):
             return self._get_object(obj_info)
 
         where = compare_columns(cls_info.primary_key, primary_vars)
@@ -940,8 +935,9 @@ class ResultSet(object):
     def config(self, distinct=None, offset=None, limit=None):
         """Configure this result object in-place. All parameters are optional.
 
-        @param distinct: Boolean enabling/disabling usage of the DISTINCT
-            keyword in the query made.
+        @param distinct: If True, enables usage of the DISTINCT keyword in
+            the query. If a tuple or list of columns, inserts a DISTINCT ON
+            (only supported by PostgreSQL).
         @param offset: Offset where results will start to be retrieved
             from the result set.
         @param limit: Limit the number of objects retrieved from the
