@@ -5,7 +5,7 @@
 """
 
 #
-# Copyright (c) 2006, 2007 Canonical
+# Copyright (c) 2006-2009 Canonical
 #
 # Written by Gustavo Niemeyer <gustavo@niemeyer.net>
 #
@@ -60,6 +60,8 @@ class ZStorm(object):
 
     implements(IZStorm)
 
+    transaction_manager = transaction.manager
+
     _databases = {}
 
     def __init__(self):
@@ -131,7 +133,8 @@ class ZStorm(object):
         store = Store(database)
         store._register_for_txn = True
         store._event.hook(
-            "register-transaction", register_store_with_transaction)
+            "register-transaction", register_store_with_transaction,
+            weakref.ref(self))
 
         self._stores[id(store)] = store
         if name is not None:
@@ -141,10 +144,12 @@ class ZStorm(object):
         return store
 
     def get(self, name, default_uri=None):
-        """Get the store called C{name} or None if one isn't available.
+        """Get the store called C{name}, creating it first if necessary.
 
         @param default_uri: Optionally, the URI to use to create a
            store called C{name} when one doesn't already exist.
+        @raises ZStormError: Raised if C{uri} is None and no default
+            URI exists for C{name}.
         """
         store = self._named.get(name)
         if not store:
@@ -174,7 +179,8 @@ class ZStorm(object):
         # Make sure the store isn't hooked up to future transactions.
         store._register_for_txn = False
         store._event.unhook(
-            "register-transaction", register_store_with_transaction)
+            "register-transaction", register_store_with_transaction,
+            weakref.ref(self))
 
     def iterstores(self):
         """Iterate C{name, store} 2-tuples."""
@@ -197,9 +203,20 @@ class ZStorm(object):
         return self._default_uris.copy()
 
 
-def register_store_with_transaction(store):
-    data_manager = StoreDataManager(store)
-    transaction.get().join(data_manager)
+def register_store_with_transaction(store, zstorm_ref):
+    zstorm = zstorm_ref()
+    if zstorm is None:
+        # zstorm object does not exist any more.
+        return False
+
+    # Check if the store is  known.  This could indicate a store being
+    # used outside of its thread.
+    if id(store) not in zstorm._stores:
+        raise ZStormError("Store not registered with ZStorm, or registered "
+                          "with another thread.")
+
+    data_manager = StoreDataManager(store, zstorm)
+    zstorm.transaction_manager.get().join(data_manager)
 
     # Unhook the event handler.  It will be rehooked for the next transaction.
     return False
@@ -210,10 +227,10 @@ class StoreDataManager(object):
 
     implements(IDataManager)
 
-    transaction_manager = transaction.manager
-
-    def __init__(self, store):
+    def __init__(self, store, zstorm):
         self._store = store
+        self._zstorm = zstorm
+        self.transaction_manager = zstorm.transaction_manager
 
     def abort(self, txn):
         try:
@@ -221,7 +238,8 @@ class StoreDataManager(object):
         finally:
             if self._store._register_for_txn:
                 self._store._event.hook(
-                    "register-transaction", register_store_with_transaction)
+                    "register-transaction", register_store_with_transaction,
+                    weakref.ref(self._zstorm))
 
     def tpc_begin(self, txn):
         # Zope's transaction system will call tpc_begin() on all
@@ -238,7 +256,8 @@ class StoreDataManager(object):
         finally:
             if self._store._register_for_txn:
                 self._store._event.hook(
-                    "register-transaction", register_store_with_transaction)
+                    "register-transaction", register_store_with_transaction,
+                    weakref.ref(self._zstorm))
 
     def tpc_vote(self, txn):
         pass
